#pragma once

#include <vector>
#define TIXML_USE_STL
#include "tinyxml.h"
#include "tipconst.h"
#include "GDIPlusImage.h"
#include "NotifyWindowItem.h"
#include "trayicon.h"
#include "ScriptletUtil.h"

#include "../../../common/TipDocument.h"
#include "../../../common/drive.h"

using namespace tipcommon;
namespace tipupdater
{
	/**
	* @class UpdaterCommonModel
	*  Ϳ  Ǵ   Ŭ
	*/
	class UpdaterCommonModel
	{
		TipDocument*  m_Document;

	public:
		UpdaterCommonModel(TipDocument* document):m_Document(document) {};		

		CString GetStartPageURL() 
		{
			CComVariant var = m_Document->GetProperty(_T("corevalue.(startpage.url)"));
			return CString(var.bstrVal);
		}
		CString GetDefaultSearchURL()
		{
			CComVariant var = m_Document->GetProperty(_T("corevalue.(defaultsearch.url)"));
			return CString(var.bstrVal);
		}
		CString GetDefaultSearchCLSID()
		{
			CComVariant var = m_Document->GetProperty(_T("corevalue.(defaultsearch.clsid)"));
			return CString(var.bstrVal);
		}
		CString GetStartPageChangedCopy()
		{
			CComVariant var = m_Document->GetProperty(_T("corevalue.(startpage.changed.copy)"));
			return CString(var.bstrVal);
		}
		CString GetDefaultSearchChangedCopy()
		{
			CComVariant var = m_Document->GetProperty(_T("corevalue.(defaultsearch.changed.copy)"));
			return CString(var.bstrVal);
		}
	};

	static const TCHAR CONFIG_TRAYICON[]					= _T("config.usetrayicon");
	static const TCHAR CONFIG_NOTIFYUPDATE[]				= _T("config.notifyupdate");
	static const TCHAR CONFIG_CHECKCOREVALUE[]				= _T("config.checkcorevalue");
	static const TCHAR CONFIG_AUTOSTART[]					= _T("config.autostart");

	class UpdaterConfigModel
	{
		TipDocument*	m_Document;
	public:
		UpdaterConfigModel(TipDocument* doc);

		void Init();
		BOOL Save();

		BOOL IsUseTrayIcon()
		{
			ATLASSERT(m_Document);
			CComVariant var = m_Document->GetProperty(CONFIG_TRAYICON);
			return ScriptUtil::Varient2BOOL(&var);
		}

		BOOL IsAutoStart()
		{
			ATLASSERT(m_Document);
			CComVariant var = m_Document->GetProperty(CONFIG_AUTOSTART);
			return ScriptUtil::Varient2BOOL(&var);
		}

		BOOL IsUseNotifyUpdate()
		{
			ATLASSERT(m_Document);
			CComVariant var = m_Document->GetProperty(CONFIG_NOTIFYUPDATE);
			return ScriptUtil::Varient2BOOL(&var);
		}

		BOOL IsUseCheckCoreValue()
		{
			ATLASSERT(m_Document);
			CComVariant var = m_Document->GetProperty(CONFIG_CHECKCOREVALUE);
			return ScriptUtil::Varient2BOOL(&var);
		}

		void SetUseTrayIcon(BOOL value)
		{
			ATLASSERT(m_Document);
			CComVariant var = value;
			m_Document->SetProperty(CONFIG_TRAYICON, var);
		}

		void SetUseAutoStart(BOOL value)
		{
			ATLASSERT(m_Document);
			CComVariant var = value;
			m_Document->SetProperty(CONFIG_AUTOSTART, var);
		}
	
		void SetUseNotifyUpdate(BOOL value)
		{
			ATLASSERT(m_Document);
			CComVariant var = value;
			m_Document->SetProperty(CONFIG_NOTIFYUPDATE, var);
		}

		void SetUseCheckCoreValue(BOOL value)
		{
			ATLASSERT(m_Document);
			CComVariant var = value;
			m_Document->SetProperty(CONFIG_CHECKCOREVALUE, var);
		}
	};

	class UpdaterNotifyWindowQueue;
	/**
	* @class UpdaterDocument
	*   ťƮ Ŭ (    )
	*/
	class UpdaterDocument : public TipDocument
	{
	public:
		UpdaterDocument();
		~UpdaterDocument(void);
		
		//override
		BOOL Open();

		/** 
		* @fn		int GetUpdaterCommonModel()
		* @brief	  뵥 ͸ ϴ Լ
		*/
		UpdaterCommonModel* GetUpdaterCommonModel() { return &m_CommonModel; }
		
		/** 
		* @fn		int GetUpdaterConfigModel()
		* @brief	  ȯ漳 ͸ ϴ Լ
		*/
		UpdaterConfigModel* GetUpdaterConfigModel() { return &m_ConfigModel; }
	
		BOOL CheckInstallInfo();

		void UpdateListCount();
		
		void SetInstalledListCount(int count)
		{
			CComVariant var = count;
			this->SetProperty(_T("installinfo.installedcount"), var);
		}
		int GetInstalledListCount()
		{ 
			CComVariant var = this->GetProperty(_T("installinfo.installedcount"));
			return var.intVal;
		}
		void SetUpdateListCount(int count)
		{
			CComVariant var = count;
			this->SetProperty(_T("updateinfo.count"), var);
		}
		int GetUpdateListCount()
		{ 
			CComVariant var = this->GetProperty(_T("updateinfo.count"));
			return var.intVal;
		}
		void SetInstallListCount(int count)
		{
			CComVariant var = count;
			this->SetProperty(_T("installinfo.count"), var);
		}
		int GetInstallListCount()
		{ 
			CComVariant var = this->GetProperty(_T("installinfo.count"));
			return var.intVal;
		}
		void ShowUpdateTab(BOOL bUpdateTab)
		{ 
			CComVariant var = bUpdateTab;
			this->SetProperty(_T("updateinfo.visible"), var);

			var = !bUpdateTab;
			this->SetProperty(_T("installinfo.visible"), var);
		}

		BOOL UpdateInstallInfo(LPCTSTR guid);

		void SetPendingService(TipServiceData* service);
		TipServiceData* GetPendingService() { return m_PendingService; }

		void SetUpdaterNotifyWindowItem( UpdaterNotifyWindowType type, const TCHAR *guid );

		void SetShowNotifyWindow(BOOL value)
		{
			m_bIsShowingNotifyWindow = value;
		}

		BOOL GetShowNotifyWindow(){ return m_bIsShowingNotifyWindow; }
		BOOL IsShowConfig(){ return m_bIsShowConfig; }
		void SetShowConfig(BOOL bShow){ m_bIsShowConfig = bShow; }
		BOOL IsUpdating(){ return (GetPendingService() != NULL); }
		UpdaterNotifyWindowQueue* GetUpdaterNotifyWindowQueue() { return m_NotifyWindowQueue; }

		BOOL GetShowAlreadyToolTipInCloseView()
		{
			return m_bShowTooltipCloseMainViewFirstTime;
		}

		void SetShowAlreadyToolTipInCloseView()
		{
			m_bShowTooltipCloseMainViewFirstTime = TRUE;
		}

		void LoadConfigModel();
		BOOL SaveConfigModel();

		void LoadCheckEventTime();
		BOOL SaveCheckEventTime();
		void InitStatManager(LPCTSTR statUrl)
		{
			m_StatManager = new UpdaterStatManager(statUrl);
		}
		CTrayIcon* GetTrayIcon();
		void InitViewProperty()
		{
			//ShowUpdateTab(TRUE);
		}
		void SetListSelect(int updateIndex, int installIndex)
		{
			this->SetProperty(_T("updateinfo.selectindex"), updateIndex);
			this->SetProperty(_T("installinfo.selectindex"), installIndex);
		}
		// ʱ Ʈ ̾ƿ ٽ ϴ  ϱ ؼ 
		BOOL IsInitIistLayout() { return m_bInitListLayout; }
		void SetInitListLayout() { m_bInitListLayout = TRUE; }
		BOOL Reload();
		LPCRITICAL_SECTION GetCriticalSection() { return &m_criticalSection; } 


	private:
		BOOL				m_bIsShowConfig;
		BOOL				m_bIsShowingNotifyWindow;
		UpdaterCommonModel m_CommonModel;
		UpdaterConfigModel m_ConfigModel;
		UpdaterNotifyWindowQueue *m_NotifyWindowQueue;
		BOOL				m_bUpdateTab;
		TipServiceData* m_PendingService;
		BOOL				m_bShowTooltipCloseMainViewFirstTime;
		BOOL				m_bInitListLayout;
		CRITICAL_SECTION	m_criticalSection;

	};
	class UpdaterXMLValidater : public TipXMLValidater
	{
	public:
		UpdaterXMLValidater();
		virtual ~UpdaterXMLValidater() 
		{
			m_TypeDefiner.RemoveAll();
		}

		// override
		BOOL IsValid(LPCTSTR key, LPCTSTR data);
	private:
		CAtlMap<CString, valueType> m_TypeDefiner;
	};
};
